"""Shot management."""


import collections

import attr
import flask_login

import pillarsdk
from pillar.web.system_util import pillar_api

from attract import attrs_extra
from attract.node_types import node_type_shot, node_type_task


@attr.s
class ShotManager(object):
    _log = attrs_extra.log('%s.ShotManager' % __name__)

    def create_shot(self, project):
        """Creates a new shot, owned by the current user.

        :rtype: pillarsdk.Node
        """

        api = pillar_api()
        node_type = project.get_node_type(node_type_shot['name'])
        if not node_type:
            raise ValueError('Project %s not set up for Attract' % project._id)

        node_props = dict(
            name='New shot',
            project=project['_id'],
            user=flask_login.current_user.objectid,
            node_type=node_type['name'],
            properties={
                'status': node_type['dyn_schema']['status']['default'],
            },
        )

        shot = pillarsdk.Node(node_props)
        shot.create(api=api)
        return shot

    def tasks_for_shots(self, shots, known_task_types):
        """Returns a dict of tasks for each shot.

        :param shots: list of shot nodes.
        :param known_task_types: Collection of task type names. Any task with a
            type not in this list will map the None key.
        :returns: a dict {shot id: tasks}, where tasks is a dict in which the keys are the
            task types, and the values are sets of tasks of that type.
        :rtype: dict

        """

        api = pillar_api()

        id_to_shot = {}
        shot_id_to_tasks = {}
        for shot in shots:
            shot_id = shot['_id']
            id_to_shot[shot_id] = shot
            shot_id_to_tasks[shot_id] = collections.defaultdict(set)

        found = pillarsdk.Node.all({
            'where': {
                'node_type': node_type_task['name'],
                'parent': {'$in': list(id_to_shot.keys())},
            }
        }, api=api)

        known = set(known_task_types)  # for fast lookups

        # Now put the tasks into the right spot.
        for task in found['_items']:
            task_type = task.properties.task_type
            if task_type not in known:
                task_type = None
            shot_id_to_tasks[task.parent][task_type].add(task)

        return shot_id_to_tasks

    def edit_shot(self, shot_id, **fields):
        """Edits a shot.

        :type shot_id: str
        :type fields: dict
        :rtype: pillarsdk.Node
        """

        api = pillar_api()
        shot = pillarsdk.Node.find(shot_id, api=api)

        shot._etag = fields.pop('_etag')
        shot.name = fields.pop('name')
        shot.description = fields.pop('description')
        shot.properties.status = fields.pop('status')
        shot.properties.notes = fields.pop('notes', '').strip() or None

        self._log.info('Saving shot %s', shot.to_dict())

        if fields:
            self._log.warning('edit_shot(%r, ...) called with unknown fields %r; ignoring them.',
                              shot_id, fields)

        shot.update(api=api)
        return shot
